// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace PlanMyNight.Models
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using PlanMyNight.Models.Entities;

    internal static class SearchHelper
    {
        public static PagingResult<T> PageResults<T>(IEnumerable<T> query, int page, int pageSize)
        {
            int rowCount = 0;
            int pageIndex = 0;
            if (pageSize > 0)
            {
                rowCount = query.Count();

                if (page > 0)
                {
                    pageIndex = page - 1;
                }

                if (page > 1)
                {
                    query = query.Skip(pageIndex * pageSize);
                }

                query = query.Take(pageSize);
            }

            var result = new PagingResult<T>(query.ToList());
            result.PageSize = pageSize;
            result.CurrentPage = pageIndex + 1;
            result.TotalItems = rowCount;

            return result;
        }

        public static IEnumerable<Activity> ActivitySearchQuery(ActivitySearchCriteria criteria, IEnumerable<Activity> activities)
        {
            IEnumerable<Activity> query = activities;

            if (criteria.ActivityTypeId.HasValue)
            {
                query = query.Where(a => a.ActivityTypeId == criteria.ActivityTypeId.Value);
            }

            string normalizedCity = criteria.City != null ? criteria.City.Trim() : null;
            if (!string.IsNullOrEmpty(normalizedCity))
            {
                query = query.Where(a => a.City.StartsWith(normalizedCity, StringComparison.OrdinalIgnoreCase));
            }

            string normalizedState = criteria.State != null ? criteria.State.Trim().ToUpperInvariant() : null;
            if (!string.IsNullOrEmpty(normalizedState))
            {
                query = query.Where(a => string.Equals(a.State, normalizedState));
            }

            string normalizedZip = criteria.Zip != null ? criteria.Zip.Trim().ToUpperInvariant() : null;
            if (!string.IsNullOrEmpty(normalizedZip))
            {
                query = query.Where(a => string.Equals(a.Zip, normalizedZip));
            }

            string normalizedAddress = criteria.StreetAddress != null ? criteria.StreetAddress.Trim().ToUpperInvariant() : null;
            if (!string.IsNullOrEmpty(normalizedAddress))
            {
                foreach (var word in normalizedAddress.Split(new[] { ' ', ',' }, StringSplitOptions.RemoveEmptyEntries))
                {
                    string wordCopy = word;
                    query = query.Where(a => a.Street.ToUpperInvariant().Contains(wordCopy));
                }
            }

            return query;
        }
    }
}