// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace PlanMyNight.Models
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using PlanMyNight.Models.Entities;

    public class ActivitiesStubRepository : IActivitiesRepository
    {
        private static IList<Activity> activities;
        private static IList<ActivityType> activityTypes;

        public ActivitiesStubRepository()
        {
            if (activities == null)
            {
                activities = new List<Activity> { 
                new Activity() { Id = "1", PhoneNumber = "12345", Name = "Roger's Cafe", ActivityTypeId = 1, State = "OH", Zip = "43215", Street = "215N Front St", City = "Columbus" },
                new Activity() { Id = "2", PhoneNumber = "12345", Name = "BD's Mongolian BBQ", ActivityTypeId = 1, State = "OH", Zip = "43215", Street = "295 Marconi Blvd", City = "Columbus" },
                new Activity() { Id = "3", PhoneNumber = "12345", Name = "Darz Café", ActivityTypeId = 1, State = "OH", Zip = "43215", Street = "111 W Nationwide Blvd", City = "Columbus" },
                new Activity() { Id = "4", PhoneNumber = "12345", Name = "Morton's of Chicago", ActivityTypeId = 1, State = "OH", Zip = "43215", Street = "2 Nationwide Plz", City = "Columbus" },
                new Activity() { Id = "5", PhoneNumber = "12345", Name = "Roger's Cafe", ActivityTypeId = 1, State = "OH", Zip = "43215", Street = "2 Nationwide Plz", City = "Columbus" },
                new Activity() { Id = "6", PhoneNumber = "12345", Name = "Fifty Five On The Blvd", ActivityTypeId = 1, State = "OH", Zip = "43215", Street = "55 W Nationwide Blvd", City = "Columbus" },
                };
            }
            if (activityTypes == null)
            {
                activityTypes = new List<ActivityType> { 
                new ActivityType() { Id = 1, Name = "Restaurant", PluralizedName = "Restaurants" }, 
                new ActivityType() { Id = 2, Name = "Bar", PluralizedName = "Bars" },
                new ActivityType() { Id = 3, Name = "Club", PluralizedName = "Clubs" }
                };
            }
        }

        public PagingResult<Activity> Search(ActivitySearchCriteria criteria)
        {
            IEnumerable<Activity> query = SearchHelper.ActivitySearchQuery(criteria, activities);

            switch (criteria.SortBy)
            {
                case SortCriteria.ActivityType:
                    query = query.OrderBy(a => a.ActivityTypeId).ThenBy(a => a.Name);
                    break;
                case SortCriteria.Rating:
                    query = query.OrderByDescending(a => a.Rating).ThenBy(a => a.Name);
                    break;
                default:
                    query = query.OrderBy(a => a.Name);
                    break;
            }

            return SearchHelper.PageResults(query, criteria.Page, criteria.PageSize);
        }

        public IEnumerable<ActivityType> RetrieveActivityTypes()
        {
            return activityTypes.ToList();
        }

        public IEnumerable<string> RetrieveStates()
        {
            return activities.Select(a => a.State)
                                 .Distinct()
                                 .ToList();
        }

        public Activity Retrieve(string id)
        {
            return activities.Where(a => a.Id == id).SingleOrDefault();
        }

        public void RateActivity(string activityId, byte rating)
        {
            var activity = activities.Where(x => x.Id == activityId).First();
            if (activity.Rating == null)
            {
                activity.Rating = rating;
                activity.RatingCount++;
            }
            else
            {
                activity.Rating = (activity.Rating * activity.RatingCount + rating) / ++activity.RatingCount;
            }
        }
    }
}