// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using PlanMyNight.Models.Entities;

namespace PlanMyNight.Models
{
    public class ActivitiesRepository : IActivitiesRepository
    {

        public Activity Retrieve(string id)
        {
            using (var context = new PlanMyNightEntities())
            {
                return context.Activities.Where(a => a.Id == id).SingleOrDefault();
            }
        }

        public PagingResult<Activity> Search(ActivitySearchCriteria criteria)
        {
            using (var context = new PlanMyNightEntities())
            {
                IEnumerable<Activity> query = SearchHelper.ActivitySearchQuery(criteria, context.Activities);

                switch (criteria.SortBy)
                {
                    case SortCriteria.ActivityType:
                        query = query.OrderBy(a => a.ActivityTypeId).ThenBy(a => a.Name);
                        break;
                    case SortCriteria.Rating:
                        query = query.OrderByDescending(a => a.Rating).ThenBy(a => a.Name);
                        break;
                    default:
                        query = query.OrderBy(a => a.Name);
                        break;
                }

                return SearchHelper.PageResults(query, criteria.Page, criteria.PageSize);
            }
        }

        public IEnumerable<ActivityType> RetrieveActivityTypes()
        {
            using (var context = new PlanMyNightEntities())
            {
                return context.ActivityTypes.ToList();
            }
        }

        public IEnumerable<string> RetrieveStates()
        {
            using (var context = new PlanMyNightEntities())
            {
                return context.Activities.Select(a => a.State).Distinct().ToList();
            }
        }

        public void RateActivity(string activityId, byte rating)
        {
            using (var context = new PlanMyNightEntities())
            {
                var activity = context.Activities.Where(a => a.Id == activityId).SingleOrDefault();

                if (activity != null)
                {
                    if (activity.Rating == null)
                    {
                        activity.Rating = rating;
                        activity.RatingCount++;
                    }
                    else
                    {
                        activity.Rating = (activity.Rating * activity.RatingCount + rating) / ++activity.RatingCount;
                    }

                    context.SaveChanges();
                }
            }
        }
    }
}