﻿var isSearching = false;

Sys.Application.add_init(function() {
    // history
    Sys.Application.set_enableHistory(true);
    Sys.Application.add_navigate(function(sender, e) {
        if (isSearching) return;

        // invoke search
        var criteria = e.get_state();
        var searchUrl = $("#searchForm form")[0].action + "?" + Sys.Application._serializeState(criteria);
        updateModel(searchUrl, onSearchRequest, onSearchResponse);

        // select fields
        selectedTypeIdOpt = $("#ActivityTypeId option[value=" + criteria.ActivityTypeId + "]");
        selectedStateOpt = $("#State option[value=" + criteria.State + "]");
        if (selectedTypeIdOpt.length == 0) selectedTypeIdOpt = $("#ActivityTypeId option:first-child");
        if (selectedStateOpt.length == 0) selectedStateOpt = $("#State option:first-child");
        selectedTypeIdOpt[0].selected = true;
        selectedStateOpt[0].selected = true;
        $("#City").val(criteria.City ? criteria.City : "");
        $("#Zip").val(criteria.Zip ? criteria.Zip : "");

        document.title = getPageTitle(criteria);
    });

    // activity templates
    $create(Sys.UI.DataView, null, null, null, $get("dynamicActivitiesItems"));         // list
    $create(Sys.UI.DataView, null, null, null, $get("dynamicActivitiesPager"));         // pager
    $create(Sys.UI.DataView, null, null, null, $get("dynamicActivitiesSort"));          // sort by

    // auto-search on change
    function autoRefresh() {
        isSearching = false;    // force cancel
        var form = $("#searchForm form");
        var url = form[0].action + "?" + form.serialize();
        updateModel(url, onSearchRequest, onSearchResponse);
    };

    $("#State").bind("change", autoRefresh);
    $("#ActivityTypeId").bind("change", autoRefresh);
    $("#City").bind("change", autoRefresh);
    $("#Zip").bind("change", autoRefresh);
    $("#StreetAddress").bind("change", autoRefresh);

    // intercept form to prevent from posting, and enable ajax calls instead
    updateModelOnFormSubmit($("#searchForm form"), onSearchRequest, onSearchResponse);

    // intercept pager links to prevent them from loading, and enable ajax calls instead
    $(".searchResults .pager a").each(function() {
        updateModelOnClickLink($(this), onSearchRequest, onSearchResponse)
    });
});

function onSearchRequest(href) {
    if (isSearching) return false;

    // validation
    var criteria = Sys.Application._deserializeState(href.split("?")[1]);
    if (!criteria || (!criteria.StreetAddress && !criteria.State && !criteria.City && !criteria.Zip)) return false;

    isSearching = true;     // flag to avoid cyclic calling when adding history point. 
    if (!criteria.page) criteria.page = null;
    if (!criteria.ActivityTypeId || criteria.ActivityTypeId == "0") criteria.ActivityTypeId = null;
    Sys.Application.addHistoryPoint(criteria, getPageTitle(criteria));

    // preloading
    $("#dynamicActivitiesEmpty").hide();
    $(".searchResults .items").addClass("loading");
    $(".searchResults .pager").addClass("disabled");
    $("#searchForm .field-validation-error").hide();
    return true;
};

function onSearchResponse(data) {
    isSearching = false;

    var orderBy = $("#searchForm input[name=SortBy]").val();
    if (orderBy == "") orderBy = "Name";

    $("#searchResultsStatic").remove();
    $("#searchResultsDynamic").show();
    $("#searchResultsDynamic h2 ul").attr("class", "active");

    // move active template to templates container
    $("#resultsTemplateContainer #dynamicActivitiesResults").remove().appendTo($("#templates"));

    var template = template = $("#dynamicActivitiesResults");
    if (data.TotalItems == 0) $("#dynamicActivitiesEmpty").show();

    // rebind items
    $find("dynamicActivitiesItems").set_data(data.Items);

    // rating labels
    showRatingLabels($("#dynamicActivitiesItems"));

    // rebind pager
    refreshPager("dynamicActivitiesPager", data);

    // rebind sort
    $find("dynamicActivitiesSort").set_data({ orderBy: orderBy, timestamp: new Date() });
    $("#dynamicActivitiesSort a").each(function(ix, el) {
        el.onclick = function() {
            var url = this.href.split("#")[0];     // strip anchors
            var sortBy = url.substring(url.indexOf('SortBy=') + 7).split('&')[0]
            $("#searchForm input[name='SortBy']").val(sortBy);
            updateModel(url, onSearchRequest, onSearchResponse, $find("dynamicSearchResults"));
            return false;
        }
    });

    // remove from templates and set into active container
    template.remove();
    $("#resultsTemplateContainer").empty().append(template);
    template.show();

    // hide loading
    $(".searchResults .items").removeClass("loading").scrollTop(0);
    $(".searchResults .items li").hide().fadeIn();
    $(".searchResults .pager").removeClass("disabled");
};

function refreshPager(pagerId, resultsData) {
    // pager
    var pages = [];
    for (var i = 1; i <= resultsData.TotalPages; i++) {
        pages.push({ pageNumber: i, currentPage: resultsData.CurrentPage, totalPages: resultsData.TotalPages });
    }

    $find(pagerId).set_data(pages);
    $("#" + pagerId + " a").each(function(ix, el) {
        el.onclick = function() {
            var url = this.href.split("#")[0];     // strip anchors
            updateModel(url, onSearchRequest, onSearchResponse);
            return false;
        }
    });
}

function showRatingLabels(container) {
    container.find("span.rating").each(function() {
        var $el = $(this);
        $el.find("span").remove();
        if ($el.text() == "0.0") {
            $el.remove();
        }
        else {
            $el.addClass("rating rating_" + $el.text().replace(".", "_"));
        }
    });
}

//AJAX Common
function updateModelOnClickLink($link, callbackRequest, callbackResponse, targetModel) {
    $link.click(function() {
        var url = this.href.split("#")[0];     // strip anchors
        updateModel(url, callbackRequest, callbackResponse, targetModel);
        return false;
    });
}

function updateModelOnFormSubmit($form, callbackRequest, callbackResponse, targetModel) {
    $form.bind("submit", function() {
        var url = this.action + '?' + $(this).serialize();
        updateModel(url, callbackRequest, callbackResponse, targetModel);
        return false;
    });
}

function updateModel(url, callbackRequest, callbackResponse, targetModel) {
    if (!callbackRequest(url)) return false;
    $.ajax({
        url: url,
        method: "GET",
        type: this.method,
        dataType: "json",
        beforeSend: function(xhr) { xhr.setRequestHeader("Content-Type", "application/json"); },
        success: function(json) {
            if (targetModel) targetModel.set_data(json);
            callbackResponse(json);
        }
    });
}

// helpers
function getPageTitle(criteria) {
    var descriptions = [];
    if (criteria.ActivityTypeId != null) descriptions.push($('#searchForm').find('#ActivityTypeId option:selected').text());
    if (criteria.City != null) descriptions.push(criteria.City);
    if (criteria.State != null) descriptions.push(criteria.State);
    if (criteria.Zip != null) descriptions.push(criteria.Zip);
    descriptions.push('Page ' + (criteria.page || "1"));
    return 'Plan My Night - Search: ' + descriptions.join(' | ');
};

function replaceQueryString(url, param, value) {
    var re = new RegExp("([?|&])" + param + "=.*?(&|$)", "i");
    if (url.match(re))
        return url.replace(re, '$1' + param + "=" + value + '$2');
    else
        return url + '&' + param + "=" + value;
}

// converters
function ratingConverter(value) {
    var val = (Math.round((value * 2)) / 2).toString();
    if (val % 1 == 0) {
        return val.toString() + ".0";
    } else {
        return val.toString()
    }
}

function activityDetailsLinkConverter(value) {
    return baseUrl + "Activities/Details/" + value;
}

function sortByConverter(value, o) {
    var $form = $("#searchForm form");
    var formAction = $form[0].action + "?" + $form.serialize();
    formAction = replaceQueryString(formAction, "SortBy", o.get_defaultValue());
    return formAction;
}

function pageConverter(value) {
    var $form = $("#searchForm form");
    return $form[0].action + "?" + $form.serialize() + "&page=" + value;
}

function previousPageConverter(value) {
    return pageConverter(value - 1);
}

function nextPageConverter(value) {
    return pageConverter(value + 1);
}