// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace PlanMyNight.ViewModels
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Text;
    using System.Web.Mvc;
    using System.Web.UI;

    public static class AjaxClientScriptExtensions
    {
        public static bool JavaScriptEnabled(this AjaxHelper ajaxHelper) {
            return System.Web.HttpContext.Current.Request.Browser.EcmaScriptVersion.Major >= 1;
        }

        public static void RegisterClientScriptInclude(this AjaxHelper ajaxHelper, string url)
        {
            var scripts = GetScriptReferences(ajaxHelper);
            if (scripts.OfType<ScriptIncludeReference>().FirstOrDefault(s => s.Url == url) == null)
            {
                scripts.Add(new ScriptIncludeReference(url));
            }
        }

        public static string RenderClientScripts(this AjaxHelper ajaxHelper)
        {
            StringBuilder sb = new StringBuilder();
            foreach (var reference in GetScriptReferences(ajaxHelper))
            {
                sb.Append(reference.Render());
            }

            return sb.ToString();
        }

        private static List<ScriptIncludeReference> GetScriptReferences(AjaxHelper ajaxHelper)
        {
            var contextItems = ajaxHelper.ViewContext.HttpContext.Items;
            if (!contextItems.Contains("AjaxClientScripts"))
            {
                contextItems["AjaxClientScripts"] = new List<ScriptIncludeReference>();
            }

            return (List<ScriptIncludeReference>)contextItems["AjaxClientScripts"];
        }

        private class ScriptIncludeReference
        {
            public ScriptIncludeReference(string url)
            {
                this.Url = url;
            }

            public string Url { get; private set; }

            public string Render()
            {
                return string.Format(CultureInfo.InvariantCulture, "<script type=\"text/javascript\" src=\"{0}\"></script>", this.Url);
            }
        }
    }
}